@@ -2,7 +2,7 @@
 @file
 Defines `boost::hana::Comparable`.
 
-@copyright Louis Dionne 2013-2016
+@copyright Louis Dionne 2013-2017
 Distributed under the Boost Software License, Version 1.0.
 (See accompanying file LICENSE.md or copy at http://boost.org/LICENSE_1_0.txt)
  */
@@ -15,15 +15,18 @@ Distributed under the Boost Software License, Version 1.0.
 #include <boost/hana/config.hpp>
 #include <boost/hana/core/default.hpp>
 #include <boost/hana/core/tag_of.hpp>
+#include <boost/hana/detail/integral_constant.hpp>
 #include <boost/hana/equal.hpp>
 
 
 BOOST_HANA_NAMESPACE_BEGIN
     template <typename T>
-    struct Comparable {
-        using Tag = typename tag_of<T>::type;
-        static constexpr bool value = !is_default<equal_impl<Tag, Tag>>::value;
-    };
+    struct Comparable
+        : hana::integral_constant<bool,
+            !is_default<equal_impl<typename tag_of<T>::type,
+                                   typename tag_of<T>::type>>::value
+        >
+    { };
 BOOST_HANA_NAMESPACE_END
 
 #endif // !BOOST_HANA_CONCEPT_COMPARABLE_HPP
