@@ -8,6 +8,7 @@
 
 #include <boost/config.hpp>
 #include <boost/scoped_array.hpp>
+#include <boost/functional/hash.hpp>
 #include <boost/multiprecision/number.hpp>
 
 #if defined(BOOST_INTEL) && !defined(BOOST_MP_USE_FLOAT128) && !defined(BOOST_MP_USE_QUAD)
@@ -118,11 +119,84 @@ using backends::float128_backend;
 
 template<>
 struct number_category<backends::float128_backend> : public mpl::int_<number_kind_floating_point> {};
+#if defined(BOOST_MP_USE_QUAD)
 template<>
 struct number_category<float128_type> : public mpl::int_<number_kind_floating_point> {};
+#endif
 
 typedef number<float128_backend, et_off> float128;
 
+#ifndef BOOST_NO_CXX11_CONSTEXPR
+
+namespace quad_constants {
+   constexpr __float128 quad_min = static_cast<__float128>(1)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN) / 1073741824;
+
+   constexpr __float128 quad_denorm_min = static_cast<__float128>(1)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN)
+      * static_cast<__float128>(DBL_MIN) / 5.5751862996326557854e+42;
+
+   constexpr double dbl_mult = 8.9884656743115795386e+307;  // This has one bit set only.
+   constexpr __float128 quad_max = (static_cast<__float128>(1) - 9.62964972193617926527988971292463659e-35)  // This now has all bits sets to 1
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult)
+      * static_cast<__float128>(dbl_mult) * 65536;
+}
+
+#define BOOST_MP_QUAD_MIN boost::multiprecision::quad_constants::quad_min
+#define BOOST_MP_QUAD_DENORM_MIN boost::multiprecision::quad_constants::quad_denorm_min
+#define BOOST_MP_QUAD_MAX boost::multiprecision::quad_constants::quad_max
+
+#else
+
+#define BOOST_MP_QUAD_MIN 3.36210314311209350626267781732175260e-4932Q
+#define BOOST_MP_QUAD_DENORM_MIN 6.475175119438025110924438958227646552e-4966Q
+#define BOOST_MP_QUAD_MAX 1.18973149535723176508575932662800702e4932Q
+
+#endif
+
 namespace backends{
 
 struct float128_backend
@@ -152,6 +226,23 @@ public:
       m_value = i;
       return *this;
    }
+   float128_backend(long double const& f)
+   {
+      BOOST_STATIC_CONSTEXPR __float128 inf_val = static_cast<__float128>(HUGE_VAL);
+      if(boost::math::isinf(f))
+         m_value = (f < 0) ? -inf_val : inf_val;
+      else
+         m_value = f;
+   }
+   float128_backend& operator=(long double const& f)
+   {
+      BOOST_STATIC_CONSTEXPR __float128 inf_val = static_cast<__float128>(HUGE_VAL);
+      if(boost::math::isinf(f))
+         m_value = (f < 0) ? -inf_val : inf_val;
+      else
+         m_value = f;
+      return *this;
+   }
    float128_backend& operator = (const char* s)
    {
 #ifndef BOOST_MP_USE_QUAD
@@ -336,7 +427,19 @@ inline void eval_sqrt(float128_backend& result, const float128_backend& arg)
 }
 inline int eval_fpclassify(const float128_backend& arg)
 {
-   return isnanq(arg.value()) ? FP_NAN : isinfq(arg.value()) ? FP_INFINITE : arg.value() == 0 ? FP_ZERO : FP_NORMAL;
+   if(isnanq(arg.value()))
+      return FP_NAN;
+   else if(isinfq(arg.value()))
+      return FP_INFINITE;
+   else if(arg.value() == 0)
+      return FP_ZERO;
+
+   float128_backend t(arg);
+   if(t.value() < 0)
+      t.negate();
+   if(t.value() < BOOST_MP_QUAD_MIN)
+      return FP_SUBNORMAL;
+   return FP_NORMAL;
 }
 
 inline void eval_increment(float128_backend& arg)
@@ -371,15 +474,6 @@ inline void eval_fabs(float128_backend& result, const float128_backend& arg)
 
 inline void eval_trunc(float128_backend& result, const float128_backend& arg)
 {
-   if(isnanq(arg.value()) || isinfq(arg.value()))
-   {
-      result = boost::math::policies::raise_rounding_error(
-            "boost::multiprecision::trunc<%1%>(%1%)", 0, 
-            number<float128_backend, et_off>(arg), 
-            number<float128_backend, et_off>(arg), 
-            boost::math::policies::policy<>()).backend();
-      return;
-   }
    result.value() = truncq(arg.value());
 }
 /*
@@ -461,10 +555,102 @@ inline void eval_atan2(float128_backend& result, const float128_backend& a, cons
 {
    result.value() = atan2q(a.value(), b.value());
 }
+#ifndef BOOST_MP_USE_QUAD
+inline void eval_multiply_add(float128_backend& result, const float128_backend& a, const float128_backend& b, const float128_backend& c)
+{
+   result.value() = fmaq(a.value(), b.value(), c.value());
+}
+inline int eval_signbit BOOST_PREVENT_MACRO_SUBSTITUTION(const float128_backend& arg)
+{
+   return ::signbitq(arg.value());
+}
+#endif
+
+inline std::size_t hash_value(const float128_backend& val)
+{
+   return  boost::hash_value(static_cast<double>(val.value()));
+}
 
 } // namespace backends
 
-}} // namespaces
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> asinh BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return asinhq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> acosh BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return acoshq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> atanh BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return atanhq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> cbrt BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return cbrtq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> erf BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return erfq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> erfc BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return erfcq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> expm1 BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return expm1q(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> lgamma BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return lgammaq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> tgamma BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return tgammaq(arg.backend().value());
+   }
+   template<boost::multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<float128_backend, ExpressionTemplates> log1p BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<float128_backend, ExpressionTemplates>& arg)
+   {
+      return log1pq(arg.backend().value());
+   }
+
+#ifndef BOOST_MP_USE_QUAD
+   template <multiprecision::expression_template_option ExpressionTemplates>
+   inline boost::multiprecision::number<boost::multiprecision::backends::float128_backend, ExpressionTemplates> copysign BOOST_PREVENT_MACRO_SUBSTITUTION(const boost::multiprecision::number<boost::multiprecision::backends::float128_backend, ExpressionTemplates>& a, const boost::multiprecision::number<boost::multiprecision::backends::float128_backend, ExpressionTemplates>& b)
+   {
+      return ::copysignq(a.backend().value(), b.backend().value());
+   }
+
+   inline void eval_remainder(float128_backend& result, const float128_backend& a, const float128_backend& b)
+   {
+      result.value() = remainderq(a.value(), b.value());
+   }
+   inline void eval_remainder(float128_backend& result, const float128_backend& a, const float128_backend& b, int* pi)
+   {
+      result.value() = remquoq(a.value(), b.value(), pi);
+   }
+#endif
+
+} // namespace multiprecision
+
+namespace math {
+
+   using boost::multiprecision::signbit;
+   using boost::multiprecision::copysign;
+
+} // namespace math
+
+} // namespace boost
 
 namespace boost{ 
 namespace archive{
@@ -520,7 +706,9 @@ void serialize(Archive& ar, boost::multiprecision::backends::float128_backend& v
    float128_detail::do_serialize(ar, val, load_tag(), binary_tag());
 }
 
-}}
+} // namepsace archive
+
+} // namespace boost
 
 namespace std{
 
@@ -530,8 +718,8 @@ class numeric_limits<boost::multiprecision::number<boost::multiprecision::backen
    typedef boost::multiprecision::number<boost::multiprecision::backends::float128_backend, ExpressionTemplates> number_type;
 public:
    BOOST_STATIC_CONSTEXPR bool is_specialized = true;
-   static number_type (min)() BOOST_NOEXCEPT { return 3.36210314311209350626267781732175260e-4932Q; }
-   static number_type (max)() BOOST_NOEXCEPT { return 1.18973149535723176508575932662800702e4932Q; }
+   static number_type (min)() BOOST_NOEXCEPT { return BOOST_MP_QUAD_MIN; }
+   static number_type (max)() BOOST_NOEXCEPT { return BOOST_MP_QUAD_MAX; }
    static number_type lowest() BOOST_NOEXCEPT { return -(max)(); }
    BOOST_STATIC_CONSTEXPR int digits = 113;
    BOOST_STATIC_CONSTEXPR int digits10 = 33;
@@ -540,7 +728,7 @@ public:
    BOOST_STATIC_CONSTEXPR bool is_integer = false;
    BOOST_STATIC_CONSTEXPR bool is_exact = false;
    BOOST_STATIC_CONSTEXPR int radix = 2;
-   static number_type epsilon() { return 1.92592994438723585305597794258492732e-34Q; }
+   static number_type epsilon() { return 1.92592994438723585305597794258492732e-34; /* this double value has only one bit set and so is exact */ }
    static number_type round_error() { return 0.5; }
    BOOST_STATIC_CONSTEXPR int min_exponent = -16381;
    BOOST_STATIC_CONSTEXPR int min_exponent10 = min_exponent * 301L / 1000L;
@@ -549,12 +737,12 @@ public:
    BOOST_STATIC_CONSTEXPR bool has_infinity = true;
    BOOST_STATIC_CONSTEXPR bool has_quiet_NaN = true;
    BOOST_STATIC_CONSTEXPR bool has_signaling_NaN = false;
-   BOOST_STATIC_CONSTEXPR float_denorm_style has_denorm = denorm_absent;
-   BOOST_STATIC_CONSTEXPR bool has_denorm_loss = false;
-   static number_type infinity() { return 1.0q / 0.0q; }
+   BOOST_STATIC_CONSTEXPR float_denorm_style has_denorm = denorm_present;
+   BOOST_STATIC_CONSTEXPR bool has_denorm_loss = true;
+   static number_type infinity() { return HUGE_VAL; /* conversion from double infinity OK */ }
    static number_type quiet_NaN() { return number_type("nan"); }
    static number_type signaling_NaN() { return 0; }
-   static number_type denorm_min() { return 0; }
+   static number_type denorm_min() { return BOOST_MP_QUAD_DENORM_MIN; }
    BOOST_STATIC_CONSTEXPR bool is_iec559 = true;
    BOOST_STATIC_CONSTEXPR bool is_bounded = false;
    BOOST_STATIC_CONSTEXPR bool is_modulo = false;
